@@ -4,32 +4,42 @@ Automate building and publishing helm charts and associated images.
 
 This is used as part of the JupyterHub and Binder projects.
 """
-
 import argparse
-from collections.abc import MutableMapping
-from functools import lru_cache, partial
 import os
 import pipes
 import re
 import shutil
 import subprocess
 import sys
+from collections.abc import MutableMapping
+from enum import Enum
+from functools import lru_cache
+from functools import partial
 from tempfile import TemporaryDirectory
 
 import docker
 from ruamel.yaml import YAML
 
-__version__ = '1.0.4'
+__version__ = "1.1.0"
 
 # name of the environment variable with GitHub token
-GITHUB_TOKEN_KEY = 'GITHUB_TOKEN'
+GITHUB_TOKEN_KEY = "GITHUB_TOKEN"
 
 # name of possible repository keys used in image value
-IMAGE_REPOSITORY_KEYS = {'name', 'repository'}
+IMAGE_REPOSITORY_KEYS = {"name", "repository"}
+
+# Container builders
+class Builder(Enum):
+    DOCKER_BUILD = "docker-build"
+    DOCKER_BUILDX = "docker-buildx"
+
+    def __str__(self):
+        return self.value
+
 
 # use safe roundtrip yaml loader capable or preserving usage of no/single/double
 # quotes for a string for example
-yaml = YAML(typ='rt')
+yaml = YAML(typ="rt")
 yaml.preserve_quotes = True
 yaml.indent(mapping=2, offset=2, sequence=4)
 
@@ -75,7 +85,9 @@ def _fix_chart_version(version, strict=False):
     """
     # semver.org provided this regular expression:
     # https://semver.org/#is-there-a-suggested-regular-expression-regex-to-check-a-semver-string
-    semver2 = re.compile(r"^(?P<major>0|[1-9]\d*)\.(?P<minor>0|[1-9]\d*)\.(?P<patch>0|[1-9]\d*)(?:-(?P<prerelease>(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$")
+    semver2 = re.compile(
+        r"^(?P<major>0|[1-9]\d*)\.(?P<minor>0|[1-9]\d*)\.(?P<patch>0|[1-9]\d*)(?:-(?P<prerelease>(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"
+    )
 
     if semver2.fullmatch(version):
         return version
@@ -99,27 +111,32 @@ def _get_git_remote_url(git_repo):
     # if not matching something/something
     # such as a local directory ".", then
     # simply return this unmodified.
-    if not re.match(r'^[^/]+/[^/]+$', git_repo):
+    if not re.match(r"^[^/]+/[^/]+$", git_repo):
         return git_repo
 
     github_token = os.getenv(GITHUB_TOKEN_KEY)
     if github_token:
-        return f'https://{github_token}@github.com/{git_repo}'
-    return f'git@github.com:{git_repo}'
+        return f"https://{github_token}@github.com/{git_repo}"
+    return f"git@github.com:{git_repo}"
 
 
 def _get_latest_commit_modifying_path(*paths, **kwargs):
     """Get the latest commit modifying the given path or return None."""
-    return _check_output(
-        [
-            'git', 'log',
-            '--max-count=1',
-            '--pretty=format:%h',
-            '--',
-            *paths,
-        ],
-        **kwargs,
-    ).decode('utf-8').strip()
+    return (
+        _check_output(
+            [
+                "git",
+                "log",
+                "--max-count=1",
+                "--pretty=format:%h",
+                "--",
+                *paths,
+            ],
+            **kwargs,
+        )
+        .decode("utf-8")
+        .strip()
+    )
 
 
 def _get_latest_tag(**kwargs):
@@ -127,22 +144,36 @@ def _get_latest_tag(**kwargs):
     try:
         # If the git command output is my-tag-14-g0aed65e,
         # then the return value will become my-tag.
-        return _check_output(
-            ['git', 'describe', '--tags', '--long'],
-            **kwargs,
-        ).decode('utf-8').strip().rsplit("-", maxsplit=2)[0]
+        return (
+            _check_output(
+                ["git", "describe", "--tags", "--long"],
+                **kwargs,
+            )
+            .decode("utf-8")
+            .strip()
+            .rsplit("-", maxsplit=2)[0]
+        )
     except subprocess.CalledProcessError:
         return None
 
 
 def _get_commit_from_tag(tag, **kwargs):
     """Return the abbreviated commit hash for the tag."""
-    return _check_output(
-        [
-            'git', 'rev-list', '--abbrev-commit', '-n', '1', tag,
-        ],
-        **kwargs,
-    ).decode('utf-8').strip()
+    return (
+        _check_output(
+            [
+                "git",
+                "rev-list",
+                "--abbrev-commit",
+                "-n",
+                "1",
+                tag,
+            ],
+            **kwargs,
+        )
+        .decode("utf-8")
+        .strip()
+    )
 
 
 @lru_cache()
@@ -163,7 +194,11 @@ def _get_latest_commit_tagged_or_modifying_paths(*paths, **kwargs):
     try:
         _check_call(
             [
-                'git', 'merge-base', '--is-ancestor', latest_commit_tagged, latest_commit_modifying_path,
+                "git",
+                "merge-base",
+                "--is-ancestor",
+                latest_commit_tagged,
+                latest_commit_modifying_path,
             ],
             **kwargs,
         )
@@ -186,7 +221,7 @@ def _get_image_build_args(image_options, ns):
         and returned, if defined.
     ns (dict): the namespace used when rendering templated arguments
     """
-    build_args = image_options.get('buildArgs', {})
+    build_args = image_options.get("buildArgs", {})
     for key, value in build_args.items():
         build_args[key] = value.format(**ns)
     return build_args
@@ -239,15 +274,24 @@ def _get_all_chart_paths(options):
     """
     paths = []
     paths.append("chartpress.yaml")
-    paths.append(options['name'])
-    paths.extend(options.get('paths', []))
-    if 'images' in options:
-        for image_name, image_config in options['images'].items():
+    paths.append(options["name"])
+    paths.extend(options.get("paths", []))
+    if "images" in options:
+        for image_name, image_config in options["images"].items():
             paths.extend(_get_all_image_paths(image_name, image_config))
     return list(set(paths))
 
 
-def build_image(image_spec, context_path, dockerfile_path=None, build_args=None):
+def build_image(
+    image_spec,
+    context_path,
+    dockerfile_path=None,
+    build_args=None,
+    *,
+    push=False,
+    builder=Builder.DOCKER_BUILD,
+    platforms=None,
+):
     """Build an image
 
     Args:
@@ -268,14 +312,45 @@ def build_image(image_spec, context_path, dockerfile_path=None, build_args=None)
         "<context_path>/Dockerfile".
     build_args (dict, optional):
         Dictionary of docker build arguments.
+    push (bool, optional):
+        Whether to push the image to a registry
+    builder (str):
+        The container build engine.
+    platforms (list[str], optional):
+        List of platforms to build for
     """
-    cmd = ['docker', 'build', '-t', image_spec, context_path]
+    if builder == Builder.DOCKER_BUILD:
+        cmd = ["docker", "build"]
+    elif builder == Builder.DOCKER_BUILDX:
+        # plain shows the container output, similar to docker build
+        cmd = ["docker", "buildx", "build", "--progress", "plain"]
+    else:
+        raise ValueError(f"Unknown builder: {builder}")
+
+    if not platforms:
+        platforms = []
+
+    cmd.extend(["-t", image_spec, context_path])
     if dockerfile_path:
-        cmd.extend(['-f', dockerfile_path])
+        cmd.extend(["-f", dockerfile_path])
     for k, v in (build_args or {}).items():
-        cmd += ['--build-arg', f'{k}={v}']
+        cmd += ["--build-arg", f"{k}={v}"]
+    if platforms:
+        cmd.extend(["--platform", ",".join(platforms)])
+    if builder == Builder.DOCKER_BUILDX:
+        # Limitations of docker buildx 0.5.1:
+        # - Can't load into the local Docker host and push to a registry at the
+        #   same time
+        # - Can't load multiple platforms into the local docker host
+        if push:
+            cmd.append("--push")
+        elif len(platforms) <= 1:
+            cmd.append("--load")
     _check_call(cmd)
 
+    if builder == Builder.DOCKER_BUILD and push:
+        _check_call(["docker", "push", image_spec])
+
 
 @lru_cache()
 def _get_docker_client():
@@ -284,7 +359,7 @@ def _get_docker_client():
 
 
 @lru_cache()
-def _image_needs_pushing(image):
+def _image_needs_pushing(image, builder):
     """
     Returns a boolean whether an image needs pushing by checking if the image
     exists on the image registry.
@@ -298,7 +373,14 @@ def _image_needs_pushing(image):
         - jupyterhub/k8s-hub:0.9.0
         - index.docker.io/library/ubuntu:latest
         - eu.gcr.io/my-gcp-project/my-image:0.1.0
+    builder (Builder):
+        The container build engine
     """
+    # docker buildx builds for multiple platforms but we can't tell which
+    # architectures have been pushed to the registry, so always push
+    if builder != Builder.DOCKER_BUILD:
+        return True
+
     d = _get_docker_client()
     try:
         d.images.get_registry_data(image)
@@ -310,7 +392,7 @@ def _image_needs_pushing(image):
 
 
 @lru_cache()
-def _image_needs_building(image):
+def _image_needs_building(image, builder):
     """
     Returns a boolean whether an image needs building by checking if the image
     exists either locally or on the registry.
@@ -324,7 +406,14 @@ def _image_needs_building(image):
         - jupyterhub/k8s-hub:0.9.0
         - index.docker.io/library/ubuntu:latest
         - eu.gcr.io/my-gcp-project/my-image:0.1.0
+    builder (Builder):
+        The container build engine
     """
+    # Since docker buildx builds for multiple platforms we can't tell whether the
+    # image already exists in the host so always build and rely on caching
+    if builder != Builder.DOCKER_BUILD:
+        return True
+
     d = _get_docker_client()
 
     # first, check for locally built image
@@ -338,20 +427,22 @@ def _image_needs_building(image):
         return False
 
     # image may need building if it's not on the registry
-    return _image_needs_pushing(image)
+    return _image_needs_pushing(image, builder)
 
 
 def _get_identifier_from_paths(*paths, long=False):
     latest_commit = _get_latest_commit_tagged_or_modifying_paths(*paths, echo=False)
 
     try:
-        git_describe = _check_output(
-            [
-                'git', 'describe', '--tags', '--long', latest_commit
-            ],
-            echo=False,
-        ).decode('utf8').strip()
-        latest_tag_in_branch, n_commits, sha = git_describe.rsplit('-', maxsplit=2)
+        git_describe = (
+            _check_output(
+                ["git", "describe", "--tags", "--long", latest_commit],
+                echo=False,
+            )
+            .decode("utf8")
+            .strip()
+        )
+        latest_tag_in_branch, n_commits, sha = git_describe.rsplit("-", maxsplit=2)
         # remove "g" prefix output by the git describe command
         # ref: https://git-scm.com/docs/git-describe#_examples
         sha = sha[1:]
@@ -360,12 +451,14 @@ def _get_identifier_from_paths(*paths, long=False):
     except subprocess.CalledProcessError:
         # no tags on branch, so assume 0.0.1 and
         # calculate n_commits from latest_commit
-        n_commits = _check_output(
-            [
-                'git', 'rev-list', '--count', latest_commit
-            ],
-            echo=False,
-        ).decode('utf-8').strip()
+        n_commits = (
+            _check_output(
+                ["git", "rev-list", "--count", latest_commit],
+                echo=False,
+            )
+            .decode("utf-8")
+            .strip()
+        )
 
         return _get_identifier_from_parts("0.0.1", n_commits, latest_commit, long)
 
@@ -417,10 +510,22 @@ def _strip_build_suffix_from_identifier(identifier):
     # split away our custom build specification: something ending in either
     # . or - followed by three or more digits, a dot, an commit sha of four
     # or more alphanumeric characters.
-    return re.sub(r'[-\.]n\d{3,}\.h\w{4,}\Z', "", identifier)
+    return re.sub(r"[-\.]n\d{3,}\.h\w{4,}\Z", "", identifier)
 
 
-def build_images(prefix, images, tag=None, push=False, force_push=False, force_build=False, skip_build=False, long=False):
+def build_images(
+    prefix,
+    images,
+    tag=None,
+    push=False,
+    force_push=False,
+    force_build=False,
+    skip_build=False,
+    long=False,
+    *,
+    builder=Builder.DOCKER_BUILD,
+    platforms=None,
+):
     """Build a collection of docker images
 
     Args:
@@ -452,6 +557,11 @@ def build_images(prefix, images, tag=None, push=False, force_push=False, force_b
         Example 2:
         - long=False: 0.9.0-n004.hsdfg2345
         - long=True:  0.9.0-n004.hsdfg2345
+
+    builder (str):
+        The container build engine.
+    platforms (list[str]):
+        List of platforms to build for if not the same as the Docker host.
     """
     values_file_modifications = {}
     for name, options in images.items():
@@ -464,25 +574,32 @@ def build_images(prefix, images, tag=None, push=False, force_push=False, force_b
         else:
             image_tag = tag
 
-        image_name = options.get('imageName', prefix + name)
+        image_name = options.get("imageName", prefix + name)
 
         # update values_file_modifications to return
-        values_path_list = options.get('valuesPath', [])
+        values_path_list = options.get("valuesPath", [])
         if isinstance(values_path_list, str):
             values_path_list = [values_path_list]
         for values_path in values_path_list:
             values_file_modifications[values_path] = {
-                'repository': image_name,
-                'tag': image_tag,
+                "repository": image_name,
+                "tag": image_tag,
             }
 
         if skip_build:
             continue
 
-        image_spec = f'{image_name}:{image_tag}'
+        image_spec = f"{image_name}:{image_tag}"
 
-        # build image
-        if force_build or _image_needs_building(image_spec):
+        skip_platforms = options.get("skipPlatforms", [])
+        if platforms and skip_platforms:
+            platforms = set(platforms).difference(skip_platforms)
+            if not platforms:
+                _log(f"Skipping build for {image_spec}, no matching platforms")
+                continue
+
+        # build image and optionally push image
+        if force_build or _image_needs_building(image_spec, builder):
             build_image(
                 image_spec,
                 _get_image_build_context_path(name, options),
@@ -490,22 +607,25 @@ def build_images(prefix, images, tag=None, push=False, force_push=False, force_b
                 build_args=_get_image_build_args(
                     options,
                     {
-                        'LAST_COMMIT': _get_latest_commit_tagged_or_modifying_paths(*all_image_paths, echo=False),
-                        'TAG': image_tag,
+                        "LAST_COMMIT": _get_latest_commit_tagged_or_modifying_paths(
+                            *all_image_paths, echo=False
+                        ),
+                        "TAG": image_tag,
                     },
-                )
+                ),
+                push=push or force_push,
+                builder=builder,
+                platforms=platforms,
             )
         else:
             _log(f"Skipping build for {image_spec}, it already exists")
 
-        # push image
-        if push or force_push:
-            if force_push or _image_needs_pushing(image_spec):
-                _check_call([
-                    'docker', 'push', image_spec
-                ])
-            else:
-                _log(f"Skipping push for {image_spec}, already on registry")
+            # push image
+            if push or force_push:
+                if force_push or _image_needs_pushing(image_spec, builder):
+                    _check_call(["docker", "push", image_spec])
+                else:
+                    _log(f"Skipping push for {image_spec}, already on registry")
 
     return values_file_modifications
 
@@ -528,17 +648,22 @@ def _update_values_file_with_modifications(name, modifications):
             }
         }
     """
-    values_file = os.path.join(name, 'values.yaml')
+    values_file = os.path.join(name, "values.yaml")
 
     with open(values_file) as f:
         values = yaml.load(f)
 
     for path_key, path_value in modifications.items():
-        if not isinstance(path_value, dict) or set(path_value.keys()) != {'repository', 'tag'}:
-            raise ValueError(f"I only understand image updates with 'repository', 'tag', not: {path_value!r}")
+        if not isinstance(path_value, dict) or set(path_value.keys()) != {
+            "repository",
+            "tag",
+        }:
+            raise ValueError(
+                f"I only understand image updates with 'repository', 'tag', not: {path_value!r}"
+            )
 
         mod_obj = parent = values
-        for p in path_key.split('.'):
+        for p in path_key.split("."):
             if p.isdigit():
                 # integers are indices in lists
                 p = int(p)
@@ -551,19 +676,21 @@ def _update_values_file_with_modifications(name, modifications):
             if keys:
                 for repo_key in keys:
                     before = mod_obj.get(repo_key, None)
-                    if before != path_value['repository']:
-                        _log(f"Updating {values_file}: {path_key}.{repo_key}: {path_value}")
-                    mod_obj[repo_key] = path_value['repository']
+                    if before != path_value["repository"]:
+                        _log(
+                            f"Updating {values_file}: {path_key}.{repo_key}: {path_value}"
+                        )
+                    mod_obj[repo_key] = path_value["repository"]
             else:
-                possible_keys = ' or '.join(IMAGE_REPOSITORY_KEYS)
+                possible_keys = " or ".join(IMAGE_REPOSITORY_KEYS)
                 raise KeyError(
-                    f'Could not find {possible_keys} in {values_file}:{path_key}'
+                    f"Could not find {possible_keys} in {values_file}:{path_key}"
                 )
 
-            before = mod_obj.get('tag', None)
-            if before != path_value['tag']:
+            before = mod_obj.get("tag", None)
+            if before != path_value["tag"]:
                 _log(f"Updating {values_file}: {path_key}.tag: {path_value}")
-            mod_obj['tag'] = path_value['tag']
+            mod_obj["tag"] = path_value["tag"]
         elif isinstance(mod_obj, str):
             # scalar image string, not dict with separate repository, tag keys
             image = "{repository}:{tag}".format(**path_value)
@@ -576,10 +703,10 @@ def _update_values_file_with_modifications(name, modifications):
             parent[last_part] = image
         else:
             raise TypeError(
-                f'{path_key} in {values_file} must be a mapping or string, not {type(mod_obj)}.'
+                f"{path_key} in {values_file} must be a mapping or string, not {type(mod_obj)}."
             )
 
-    with open(values_file, 'w') as f:
+    with open(values_file, "w") as f:
         yaml.dump(values, f)
 
 
@@ -602,7 +729,7 @@ def build_chart(
         - 0.9.0-n002.hdfgh3456
     """
     # read Chart.yaml
-    chart_file = os.path.join(name, 'Chart.yaml')
+    chart_file = os.path.join(name, "Chart.yaml")
     with open(chart_file) as f:
         chart = yaml.load(f)
 
@@ -613,17 +740,24 @@ def build_chart(
         version = _fix_chart_version(version, strict=strict_version)
 
     # update Chart.yaml
-    if chart['version'] != version:
+    if chart["version"] != version:
         _log(f"Updating {chart_file}: version: {version}")
-        chart['version'] = version
-    with open(chart_file, 'w') as f:
+        chart["version"] = version
+    with open(chart_file, "w") as f:
         yaml.dump(chart, f)
 
     # return version
     return version
 
 
-def publish_pages(chart_name, chart_version, chart_repo_github_path, chart_repo_url, extra_message="", force=False):
+def publish_pages(
+    chart_name,
+    chart_version,
+    chart_repo_github_path,
+    chart_repo_url,
+    extra_message="",
+    force=False,
+):
     """
     Update a Helm chart registry hosted in the gh-pages branch of a GitHub git
     repository.
@@ -656,11 +790,13 @@ def publish_pages(chart_name, chart_version, chart_repo_github_path, chart_repo_
 
     # clone/fetch the Helm chart repo and checkout its gh-pages branch, note the
     # use of cwd (current working directory)
-    checkout_dir = f'{chart_name}-{chart_version}'
+    checkout_dir = f"{chart_name}-{chart_version}"
     if not os.path.isdir(checkout_dir):
         _check_call(
             [
-                'git', 'clone', '--no-checkout',
+                "git",
+                "clone",
+                "--no-checkout",
                 _get_git_remote_url(chart_repo_github_path),
                 checkout_dir,
             ],
@@ -668,61 +804,76 @@ def publish_pages(chart_name, chart_version, chart_repo_github_path, chart_repo_
             echo=True,
         )
     else:
-        _check_call(['git', 'fetch'], cwd=checkout_dir, echo=True)
-    _check_call(['git', 'checkout', 'gh-pages'], cwd=checkout_dir, echo=True)
+        _check_call(["git", "fetch"], cwd=checkout_dir, echo=True)
+    _check_call(["git", "checkout", "gh-pages"], cwd=checkout_dir, echo=True)
 
     # check if a chart with the same name and version has already been published. If
     # there is, the behaviour depends on `-force-publish-chart`
     # and chart_version and make a decision based on the --force-publish-chart
     # flag if that is the case, but always log what's done
-    if os.path.isfile(os.path.join(checkout_dir, 'index.yaml')):
-        with open(os.path.join(checkout_dir, 'index.yaml')) as f:
+    if os.path.isfile(os.path.join(checkout_dir, "index.yaml")):
+        with open(os.path.join(checkout_dir, "index.yaml")) as f:
             chart_repo_index = yaml.load(f)
             published_charts = chart_repo_index["entries"].get(chart_name)
 
         if any(c["version"] == chart_version for c in published_charts):
             if force:
-                _log(f"Chart of version {chart_version} already exists, overwriting it.")
+                _log(
+                    f"Chart of version {chart_version} already exists, overwriting it."
+                )
             else:
-                _log(f"Skipping chart publishing of version {chart_version}, it is already published")
+                _log(
+                    f"Skipping chart publishing of version {chart_version}, it is already published"
+                )
                 return
 
     # package the latest version into a temporary directory
     # and run helm repo index with --merge to update index.yaml
     # without refreshing all of the timestamps
     with TemporaryDirectory() as td:
-        _check_call([
-            'helm', 'package', chart_name,
-            '--dependency-update',
-            '--destination', td + '/',
-        ])
+        _check_call(
+            [
+                "helm",
+                "package",
+                chart_name,
+                "--dependency-update",
+                "--destination",
+                td + "/",
+            ]
+        )
 
-        _check_call([
-            'helm', 'repo', 'index', td,
-            '--url', chart_repo_url,
-            '--merge', os.path.join(checkout_dir, 'index.yaml'),
-        ])
+        _check_call(
+            [
+                "helm",
+                "repo",
+                "index",
+                td,
+                "--url",
+                chart_repo_url,
+                "--merge",
+                os.path.join(checkout_dir, "index.yaml"),
+            ]
+        )
 
         # equivalent to `cp td/* checkout/`
         # copies new helm chart and updated index.yaml
         for f in os.listdir(td):
-            shutil.copy2(
-                os.path.join(td, f),
-                os.path.join(checkout_dir, f)
-            )
+            shutil.copy2(os.path.join(td, f), os.path.join(checkout_dir, f))
 
     # git add, commit, and push
-    extra_message = f'\n\n{extra_message}' if extra_message else ''
-    message = f'[{chart_name}] Automatic update for commit {chart_version}{extra_message}'
-
-    _check_call(['git', 'add', '.'], cwd=checkout_dir)
-    _check_call(['git', 'commit', '-m', message], cwd=checkout_dir)
-    _check_call(['git', 'push', 'origin', 'gh-pages'], cwd=checkout_dir)
+    extra_message = f"\n\n{extra_message}" if extra_message else ""
+    message = (
+        f"[{chart_name}] Automatic update for commit {chart_version}{extra_message}"
+    )
 
+    _check_call(["git", "add", "."], cwd=checkout_dir)
+    _check_call(["git", "commit", "-m", message], cwd=checkout_dir)
+    _check_call(["git", "push", "origin", "gh-pages"], cwd=checkout_dir)
 
 
 class ActionStoreDeprecated(argparse.Action):
     """Used with argparse as a deprecation action."""
+
     def __call__(self, parser, namespace, values, option_string=None):
         _log(f"Warning: use of {'|'.join(self.option_strings)} is deprecated.")
         setattr(namespace, self.dest, values)
@@ -730,6 +881,7 @@ class ActionStoreDeprecated(argparse.Action):
 
 class ActionAppendDeprecated(argparse.Action):
     """Used with argparse as a deprecation action."""
+
     def __call__(self, parser, namespace, values, option_string=None):
         _log(f"Warning: use of {'|'.join(self.option_strings)} is deprecated.")
         if not getattr(namespace, self.dest):
@@ -737,55 +889,54 @@ class ActionAppendDeprecated(argparse.Action):
         getattr(namespace, self.dest).append(values)
 
 
-
 def main(args=None):
     """Run chartpress"""
     argparser = argparse.ArgumentParser(description=__doc__)
 
     argparser.add_argument(
-        '--push',
-        action='store_true',
-        help='Push built images to their image registries, but not if it would replace an existing image.',
+        "--push",
+        action="store_true",
+        help="Push built images to their image registries, but not if it would replace an existing image.",
     )
     argparser.add_argument(
-        '--force-push',
-        action='store_true',
-        help='Push built images to their image registries, regardless if it would replace an existing image.',
+        "--force-push",
+        action="store_true",
+        help="Push built images to their image registries, regardless if it would replace an existing image.",
     )
     argparser.add_argument(
-        '--publish-chart',
-        action='store_true',
-        help='Package a Helm chart and publish it to a Helm chart registry constructed with a GitHub git repository and GitHub pages, but not if it would replace an existing chart version.',
+        "--publish-chart",
+        action="store_true",
+        help="Package a Helm chart and publish it to a Helm chart registry constructed with a GitHub git repository and GitHub pages, but not if it would replace an existing chart version.",
     )
     argparser.add_argument(
-        '--force-publish-chart',
-        action='store_true',
-        help='Package a Helm chart and publish it to a Helm chart registry constructed with a GitHub git repository and GitHub pages, regardless if it would replace an existing chart version',
+        "--force-publish-chart",
+        action="store_true",
+        help="Package a Helm chart and publish it to a Helm chart registry constructed with a GitHub git repository and GitHub pages, regardless if it would replace an existing chart version",
     )
     argparser.add_argument(
-        '--extra-message',
-        default='',
-        help='Extra message to add to the commit message when publishing charts.',
+        "--extra-message",
+        default="",
+        help="Extra message to add to the commit message when publishing charts.",
     )
     tag_or_long_group = argparser.add_mutually_exclusive_group()
     tag_or_long_group.add_argument(
-        '--tag',
+        "--tag",
         default=None,
         help="Explicitly set the image tags and chart version.",
     )
     tag_or_long_group.add_argument(
-        '--long',
-        action='store_true',
-        help='Use this to always get a build suffix for the generated tag and chart version, even when the specific commit has a tag.',
+        "--long",
+        action="store_true",
+        help="Use this to always get a build suffix for the generated tag and chart version, even when the specific commit has a tag.",
     )
     argparser.add_argument(
-        '--image-prefix',
+        "--image-prefix",
         default=None,
-        help='Override the configured image prefix with this value.',
+        help="Override the configured image prefix with this value.",
     )
     argparser.add_argument(
-        '--reset',
-        action='store_true',
+        "--reset",
+        action="store_true",
         help="Skip image build step and reset Chart.yaml's version field and values.yaml's image tags. What it resets to can be configured in chartpress.yaml with the resetTag and resetVersion configurations.",
     )
     skip_or_force_build_group = argparser.add_mutually_exclusive_group()
@@ -796,9 +947,28 @@ def main(args=None):
         help="Skip the image build step.",
     )
     skip_or_force_build_group.add_argument(
-        '--force-build',
-        action='store_true',
-        help='Enforce the image build step, regardless of if the image already is available either locally or remotely.',
+        "--force-build",
+        action="store_true",
+        help="Enforce the image build step, regardless of if the image already is available either locally or remotely.",
+    )
+
+    argparser.add_argument(
+        "--builder",
+        choices=list(Builder),
+        default=Builder.DOCKER_BUILD,
+        type=Builder,
+        help=f"Container build engine to use, {Builder.DOCKER_BUILD} is the standard Docker build command.",
+    )
+
+    argparser.add_argument(
+        "--platform",
+        action="append",
+        help=(
+            "Build the image for this platform, e.g. linux/amd64 or linux/arm64. "
+            "This argument can be used multiple times to build multiple platforms under the same tag. "
+            "Only supported for docker buildx. "
+            "If --push is set or if multiple platforms are passed the image will not be loaded into the local docker engine."
+        ),
     )
 
     argparser.add_argument(
@@ -807,12 +977,15 @@ def main(args=None):
         help="print list of images to stdout. Images will not be built.",
     )
     argparser.add_argument(
-        '--version',
-        action='store_true',
-        help='Print current chartpress version and exit.',
+        "--version",
+        action="store_true",
+        help="Print current chartpress version and exit.",
     )
 
     args = argparser.parse_args(args)
+    if args.builder == Builder.DOCKER_BUILD and args.platform:
+        argparser.error(f"--platform is not supported with {Builder.DOCKER_BUILD}")
+
     # allow simple checks for whether publish will happen
     if args.force_publish_chart:
         args.publish_chart = True
@@ -824,8 +997,7 @@ def main(args=None):
     if args.list_images:
         args.no_build = True
 
-
-    with open('chartpress.yaml') as f:
+    with open("chartpress.yaml") as f:
         config = yaml.load(f)
 
     # main logic
@@ -835,17 +1007,17 @@ def main(args=None):
     #     - push images (--push)
     #   - build values.yaml (--reset)
     #   - push chart (--publish-chart, --extra-message)
-    for chart in config['charts']:
+    for chart in config["charts"]:
         forced_version = None
         if args.tag:
             forced_version = args.tag
         if args.reset:
-            forced_version = chart.get('resetVersion', '0.0.1-set.by.chartpress')
+            forced_version = chart.get("resetVersion", "0.0.1-set.by.chartpress")
 
         if not args.list_images:
             # update Chart.yaml with a version
             chart_version = build_chart(
-                chart['name'],
+                chart["name"],
                 paths=_get_all_chart_paths(chart),
                 version=forced_version,
                 long=args.long,
@@ -853,17 +1025,21 @@ def main(args=None):
                 strict_version=args.publish_chart,
             )
 
-        if 'images' in chart:
+        if "images" in chart:
             # build images
             values_file_modifications = build_images(
-                prefix=args.image_prefix or chart.get('imagePrefix', ''),
-                images=chart['images'],
-                tag=args.tag if not args.reset else chart.get('resetTag', 'set-by-chartpress'),
+                prefix=args.image_prefix or chart.get("imagePrefix", ""),
+                images=chart["images"],
+                tag=args.tag
+                if not args.reset
+                else chart.get("resetTag", "set-by-chartpress"),
                 push=args.push,
                 force_push=args.force_push,
                 force_build=args.force_build,
                 skip_build=args.no_build or args.reset,
                 long=args.long,
+                builder=args.builder,
+                platforms=args.platform,
             )
 
             # list images
@@ -878,19 +1054,21 @@ def main(args=None):
                 return
 
             # update values.yaml
-            _update_values_file_with_modifications(chart['name'], values_file_modifications)
+            _update_values_file_with_modifications(
+                chart["name"], values_file_modifications
+            )
 
         # publish chart
         if args.publish_chart:
             publish_pages(
-                chart_name=chart['name'],
+                chart_name=chart["name"],
                 chart_version=chart_version,
-                chart_repo_github_path=chart['repo']['git'],
-                chart_repo_url=chart['repo']['published'],
+                chart_repo_github_path=chart["repo"]["git"],
+                chart_repo_url=chart["repo"]["published"],
                 extra_message=args.extra_message,
                 force=args.force_publish_chart,
             )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
